#!/usr/bin/env python

#Creates a torn old looking paper (GIMP plug-in)
#Copyright (c) 2008 by Sagenlicht from Cartographers Guild
#http://cartographersguild.com

# Version 1.2
# Changes/fixes are found in the change.log

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from gimpfu import *
import random

def tornOldPaper(img, draw, paperColor, createRandomPaper, shrinkSize, spread, granularity, smooth, addShadow):
	#Variables
	threshold = 127
	vertSmooth = True
	horSmooth = True
	seedTexture = random.randrange(1,999999999)
	turbulenceTexture = 2.5
	seedFuzz = random.randrange(1,999999999)
	turbulenceFuzz = 2.5
	bumpAzimuth = 135.0
	bumpElevation = 45.0
	bumpDepth = 5
	bumpOffX = 0
	bumpOffY = 0
	bumpWaterLvl = 0
	bumpAmbient = 0
	bumpCompensate = 1
	bumpInvert = 0
	bumpType = 0
	shadowOffX = 5
	shadowOffY = 5
	shadowBlur = 15
	shadowColor = gimpcolor.RGB(0,0,0)
	shadowOpacity = 60
	shadowAllowResize = 1
	
	#Debug variables
	
	#Set Undo Start
	pdb.gimp_image_undo_group_start(img)
	
	#Saving fore- and background color
	origForeColor = pdb.gimp_context_get_foreground()
	origBackColor = pdb.gimp_context_get_background()
	
	#Setting fore- and background color
	gimp.set_foreground(paperColor)
	
	#Selections
	pdb.gimp_selection_all(img)
	allSelection = pdb.gimp_selection_save(img)
	
	if createRandomPaper:
		pdb.gimp_selection_shrink(img,shrinkSize)
		pdb.script_fu_distress_selection(img, draw, threshold, spread, granularity, \
				smooth, vertSmooth, horSmooth)
		paperSelection = pdb.gimp_selection_save(img)
	else:
		pdb.gimp_selection_none(img)
		pdb.gimp_by_color_select(draw, (0,0,0), 15, 0, TRUE, FALSE, 0, FALSE)
		pdb.gimp_selection_grow(img, 1)
		paperSelection = pdb.gimp_selection_save(img)
		
	#Adding paperLayer and fill it with paperColor
	paperLayer = gimp.Layer(img, "Paper", img.width, img.height, (RGBA_IMAGE, GRAYA_IMAGE)[img.base_type], 100.0, NORMAL_MODE)
	img.add_layer(paperLayer, 0)
	pdb.gimp_selection_load(allSelection)
	pdb.gimp_edit_clear(paperLayer)
	pdb.gimp_selection_load(paperSelection)
	pdb.gimp_edit_fill(paperLayer, FOREGROUND_FILL)

	#Adding textureLayer in overlay mode
	textureLayer = gimp.Layer(img, "Texture", img.width, img.height, (RGBA_IMAGE, GRAYA_IMAGE)[img.base_type], 100.0, OVERLAY_MODE)
	img.add_layer(textureLayer, 0)
	pdb.gimp_selection_load(allSelection)
	pdb.gimp_edit_clear(textureLayer)
	pdb.gimp_selection_load(paperSelection)
	
	#Fill textureLayer with plasma plasma and desturate it
	pdb.plug_in_plasma(img, textureLayer, seedTexture, turbulenceTexture)
	pdb.gimp_desaturate_full(textureLayer, 0)
	
	#Bump the plasma with itself
	pdb.plug_in_bump_map(img, textureLayer, textureLayer, bumpAzimuth, bumpElevation, bumpDepth, bumpOffX, bumpOffY, bumpWaterLvl, \
			bumpAmbient, bumpCompensate, bumpInvert, bumpType)

	#Adding blotsLayer overlay mode - must be filled manually
	blotsLayer = gimp.Layer(img, "Blots", img.width, img.height, (RGBA_IMAGE, GRAYA_IMAGE)[img.base_type], 100.0, OVERLAY_MODE)
	img.add_layer(blotsLayer, 0)
	pdb.gimp_selection_load(allSelection)
	pdb.gimp_edit_clear(blotsLayer)
	pdb.gimp_selection_load(paperSelection)
	
	#Adding fuzzLayer in overlay mode
	fuzzLayer = gimp.Layer(img, "Fuzz", img.width, img.height, (RGBA_IMAGE, GRAYA_IMAGE)[img.base_type], 100.0, OVERLAY_MODE)
	img.add_layer(fuzzLayer, 0)
	pdb.gimp_selection_load(allSelection)
	pdb.gimp_edit_clear(fuzzLayer)
	pdb.gimp_selection_load(paperSelection)
	
	#Fill the fuzzlayer with plasma and desturate it
	pdb.plug_in_plasma(img, fuzzLayer, seedFuzz, turbulenceFuzz)
	pdb.gimp_desaturate_full(fuzzLayer, 0)
	
	#Adding shadow to the paper if selected
	if addShadow:
		pdb.script_fu_drop_shadow(img, fuzzLayer, shadowOffX, shadowOffY, shadowBlur, shadowColor, shadowOpacity, shadowAllowResize)
	
	#Adding txtLayer in overlay mode - its more a placeholder
	txtLayer = gimp.Layer(img, "Add your text or symbols here", img.width, img.height, (RGBA_IMAGE, GRAYA_IMAGE)[img.base_type], 100.0, OVERLAY_MODE)
	img.add_layer(txtLayer, 2)
	pdb.gimp_selection_load(allSelection)
	pdb.gimp_edit_clear(txtLayer)
	
	#Discard selection
	pdb.gimp_selection_none(img)
	
	#Restore original fore- and background color
	pdb.gimp_context_set_foreground(origForeColor)
	pdb.gimp_context_set_background(origBackColor)
	
	#Set Undo End
	pdb.gimp_image_undo_group_end(img)


register(
	"python_fu_tornOldPaper",
	"Creates a torn old paper",
	"Creates a torn old paper",
	"Christian Kremer",
	"Christian Kremer",
	"2008",
   "<Image>/Filters/RPG Filters/Torn Old Paper",
	"RGB*, GRAY*",
	[
		(PF_COLOR, "paperColor", "Document Color", (208,193,162)),
		(PF_BOOL, "createRandomPaper", "Create a random paper?", TRUE),
		(PF_INT, "shrinkSize", "Reduce selection by pixels", 40),
		(PF_INT, "spread", "Distress Spread ", 10),
		(PF_INT, "granularity", "Distress Granularity", 6),
		(PF_INT, "smooth", "Distress Smooth", 2),
		(PF_BOOL, "addShadow", "Add shadow to paper?", TRUE),
	],
	[],
	tornOldPaper)


main()

